
:root {
    --textColor: {{ theme_override_color.textColor }};
    --linkColor: {{ theme_override_color.linkColor }};
    --linkHoverColor: {{ theme_override_color.linkHoverColor }};
    --linkVisitedColor: {{ theme_override_color.linkVisitedColor }};
    --bgLevel1: {{ theme_override_color.bgColor1 }};
    --bgLevel2: {{ theme_override_color.bgColor2 }};
    --bgLevel3: {{ theme_override_color.bgColor3 }};
}

body {
    background-color: var(--bgLevel1);
}

a {
    color: var(--linkColor);
}

a:hover {
    color: var(--linkHoverColor);
}

a:visited {
    color: var(--linkVisitedColor);
}

body,
body table td,
body table th,
.footer,
.breadcrumb li.active,
.breadcrumb li.active a,
.left-nav > ul > li > a,
.left-nav > ul > li ul li.active a {
    color: var(--textColor);
}

body table td,
body table th {
    text-shadow: none !important;
}

#wrap {
    background: transparent;
}

.login #content-main h1,
.login #content-main .submit-row,
.form-horizontal .control-group .controls,
.breadcrumb,
.box, .well,
.left-nav > ul > li.active > a,
.left-nav > ul > li ul li a:hover,
.footer,
.header,
body table td,
body table th {
    background: var(--bgLevel2)!important;
}

.login #content-main,
.form-horizontal .control-group .control-label,
.left-nav > ul > li ul {
    background-color: var(--bgLevel3);
}

.login #content-main .submit-row {
    border-top: none;
}

.breadcrumb > li {
    text-shadow: none;
}

.left-nav > ul > li ul li a:hover {
    text-shadow: none;
}